package edu.washington.cs.radishsim;

import java.io.IOException;
import java.io.Writer;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

public class Counter {
   private CpuId                cpuid;
   private long                 stat;
   private String               name;

   /** List of all the stats that have been created. */
   private static List<Counter> AllCounters = new LinkedList<Counter>();

   /**
    * Each counter is associated with a CPU. New Counters are associated with whatever this field's
    * value is at the time of their construction. This is a hack to avoid having to pass a CpuId to
    * the Counter ctor, which breaks direct initialization of Counter fields (and requires the init
    * to happen in a ctor instead, duplicating all the Counter field names).
    */
   public static CpuId          currentCpu;

   public static void dumpCounters(Writer wr, String prefix, String suffix) throws IOException {
      
      // generate global counters
      
      // maps pc counter names => global Counter objects
      currentCpu = new CpuId(-1);
      Map<String, Counter> globalCounters = new HashMap<String, Counter>();
      for ( Counter c : AllCounters ) {
         if ( !c.name.startsWith("pc_") ) continue;
         
         if ( globalCounters.containsKey(c.name) ) {
            Counter g = globalCounters.get(c.name);
            g.set(g.get() + c.get());
         } else {
            Counter g = new Counter(c.name.replace("pc_", "g_"), true);
            g.set(c.get());
            globalCounters.put(c.name, g);
         }
      }
      currentCpu = null;
      AllCounters.addAll(globalCounters.values());
      
      // write counter values
      for ( Counter c : AllCounters ) {
         wr.write(prefix + "'cpuid': " + c.cpuid.get() + ", '" + c.name + "': " + c.stat + suffix);
      }
   }

   private Counter(String name, boolean _) {
      assert currentCpu != null;
      this.cpuid = currentCpu;
      this.name = name;
   }
   
   Counter(String name) {
      AllCounters.add(this);
      assert currentCpu != null;
      this.cpuid = currentCpu;
      this.name = name;
   }

   public long get() {
      return stat;
   }

   public void set(long v) {
      stat = v;
   }

   public void incr() {
      stat++;
   }
   
   public void incr(long a) {
      stat += a;
   }
}